@@ -1,6 +1,7 @@
 package engine
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -10,6 +11,7 @@ import (
 
 	"github.com/spf13/viper"
 	"golang.org/x/time/rate"
+	"gopkg.in/yaml.v2"
 )
 
 const (
@@ -26,41 +28,45 @@ const (
 )
 
 type Config struct {
-	AutoStart               bool
-	EngineDebug             bool
-	MuteEngineLog           bool
-	ObfsPreferred           bool
-	ObfsRequirePreferred    bool
-	DisableTrackers         bool
-	DisableIPv6             bool
-	NoDefaultPortForwarding bool
-	DisableUTP              bool
-	DownloadDirectory       string
-	WatchDirectory          string
-	EnableUpload            bool
-	EnableSeeding           bool
-	IncomingPort            int
-	DoneCmd                 string
-	SeedRatio               float32
-	SeedTime                time.Duration
-	UploadRate              string
-	DownloadRate            string
-	TrackerList             string
-	AlwaysAddTrackers       bool
-	ProxyURL                string
-	RssURL                  string
-	ScraperURL              string
-	MaxConcurrentTask       int
-	AllowRuntimeConfigure   bool
+	AutoStart               bool          `yaml:"AutoStart"`
+	EngineDebug             bool          `yaml:"EngineDebug"`
+	MuteEngineLog           bool          `yaml:"MuteEngineLog"`
+	ObfsPreferred           bool          `yaml:"ObfsPreferred"`
+	ObfsRequirePreferred    bool          `yaml:"ObfsRequirePreferred"`
+	DisableTrackers         bool          `yaml:"DisableTrackers"`
+	DisableIPv6             bool          `yaml:"DisableIPv6"`
+	NoDefaultPortForwarding bool          `yaml:"NoDefaultPortForwarding"`
+	DisableUTP              bool          `yaml:"DisableUTP"`
+	DownloadDirectory       string        `yaml:"DownloadDirectory"`
+	WatchDirectory          string        `yaml:"WatchDirectory"`
+	EnableUpload            bool          `yaml:"EnableUpload"`
+	EnableSeeding           bool          `yaml:"EnableSeeding"`
+	IncomingPort            int           `yaml:"IncomingPort"`
+	DoneCmd                 string        `yaml:"DoneCmd"`
+	SeedRatio               float32       `yaml:"SeedRatio"`
+	SeedTime                time.Duration `yaml:"SeedTime"`
+	UploadRate              string        `yaml:"UploadRate"`
+	DownloadRate            string        `yaml:"DownloadRate"`
+	TrackerList             string        `yaml:"TrackerList"`
+	AlwaysAddTrackers       bool          `yaml:"AlwaysAddTrackers"`
+	ProxyURL                string        `yaml:"ProxyURL"`
+	RssURL                  string        `yaml:"RssURL"`
+	ScraperURL              string        `yaml:"ScraperURL"`
+	MaxConcurrentTask       int           `yaml:"MaxConcurrentTask"`
+	AllowRuntimeConfigure   bool          `yaml:"AllowRuntimeConfigure"`
 }
 
-func InitConf(specPath string) (*Config, error) {
-
-	viper.SetConfigName("cloud-torrent")
-	viper.AddConfigPath("/etc/cloud-torrent/")
-	viper.AddConfigPath("/etc/")
-	viper.AddConfigPath("$HOME/.cloud-torrent")
-	viper.AddConfigPath(".")
+func InitConf(specPath *string) (*Config, error) {
+	if *specPath != "" {
+		// user specific config path
+		viper.SetConfigFile(*specPath)
+	} else {
+		viper.SetConfigName("cloud-torrent")
+		viper.AddConfigPath("/etc/cloud-torrent/")
+		viper.AddConfigPath("/etc/")
+		viper.AddConfigPath("$HOME/.cloud-torrent")
+		viper.AddConfigPath(".")
+	}
 
 	viper.SetDefault("DownloadDirectory", "./downloads")
 	viper.SetDefault("WatchDirectory", "./torrents")
@@ -78,24 +84,19 @@ func InitConf(specPath string) (*Config, error) {
 	viper.SetDefault("MaxConcurrentTask", 0)
 	viper.SetDefault("AllowRuntimeConfigure", true)
 
-	// user specific config path
-	if stat, err := os.Stat(specPath); stat != nil && err == nil {
-		viper.SetConfigFile(specPath)
-	}
-
 	configExists := true
 	if err := viper.ReadInConfig(); err != nil {
-		if strings.Contains(err.Error(), "Not Found") {
+		if errors.Is(err, os.ErrNotExist) {
+			// user set a config that is not exists, will write to it later
 			configExists = false
-			if specPath == "" {
-				specPath = "./cloud-torrent.yaml"
-			}
-			viper.SetConfigFile(specPath)
 		} else {
 			return nil, err
 		}
 	}
 
+	*specPath = viper.ConfigFileUsed()
+	log.Println("[config] using config file: ", *specPath)
+
 	c := &Config{}
 	viper.Unmarshal(c)
 
@@ -108,13 +109,11 @@ func InitConf(specPath string) (*Config, error) {
 		viper.Set("WatchDirectory", c.WatchDirectory)
 	}
 
-	cf := viper.ConfigFileUsed()
-	log.Println("[config] selected config file: ", cf)
 	if !configExists || dirChanged {
-		if err := viper.WriteConfig(); err != nil {
+		if err := c.WriteDefault(); err != nil {
 			return nil, err
 		}
-		log.Println("[config] config file written: ", cf, "exists:", configExists, "dirchanged", dirChanged)
+		log.Println("[config] config file updated: ", *specPath, "exists:", configExists, "dirchanged", dirChanged)
 	}
 
 	return c, nil
@@ -207,7 +206,7 @@ func (c *Config) Validate(nc *Config) uint8 {
 	return status
 }
 
-func (c *Config) SyncViper(nc Config) error {
+func (c *Config) SyncViper(nc Config) {
 	cv := reflect.ValueOf(*c)
 	nv := reflect.ValueOf(nc)
 	typeOfC := cv.Type()
@@ -220,10 +219,24 @@ func (c *Config) SyncViper(nc Config) error {
 			log.Println("config updated ", name, ": ", oval, " -> ", val)
 		}
 	}
+}
 
+func (c *Config) WriteDefault() error {
+	cf := viper.ConfigFileUsed()
+	if strings.HasSuffix(cf, ".yml") || strings.HasSuffix(cf, ".yaml") {
+		return c.WriteYaml(cf)
+	}
 	return viper.WriteConfig()
 }
 
+func (c *Config) WriteYaml(cf string) error {
+	d, err := yaml.Marshal(c)
+	if err != nil {
+		return err
+	}
+	return os.WriteFile(cf, d, 0666)
+}
+
 func (c *Config) GetCmdConfig() (string, []string, error) {
 	if c.DoneCmd == "" {
 		return "", nil, fmt.Errorf("unconfigred Donecmd")
