@@ -10,8 +10,7 @@ import (
 )
 
 type Torrent struct {
-	// put at first postition to prevent memorty align issues.
-	Stats torrent.TorrentStats
+	sync.Mutex
 
 	//anacrolix/torrent
 	InfoHash   string
@@ -24,6 +23,7 @@ type Torrent struct {
 	Files      []*File
 
 	//cloud torrent
+	Stats          *torrent.TorrentStats
 	Started        bool
 	Done           bool
 	DoneCmdCalled  bool
@@ -44,7 +44,6 @@ type Torrent struct {
 	e              *Engine
 	dropWait       chan struct{}
 	cld            Server
-	sync.Mutex
 }
 
 type File struct {
@@ -85,35 +84,34 @@ func (torrent *Torrent) updateOnGotInfo(t *torrent.Torrent) {
 }
 
 func (torrent *Torrent) updateConnStat() {
-	torrent.Stats = torrent.t.Stats()
+	lastStat := torrent.Stats
+	curStat := torrent.t.Stats()
+	now := time.Now()
 
 	// calculate ratio
-	bRead := torrent.Stats.BytesReadData.Int64()
-	bWrite := torrent.Stats.BytesWrittenData.Int64()
+	bRead := curStat.BytesReadUsefulData.Int64()
+	bWrite := curStat.BytesWrittenData.Int64()
 	if bRead > 0 {
 		torrent.SeedRatio = float32(bWrite) / float32(bRead)
 	} else if torrent.Done {
 		torrent.SeedRatio = float32(bWrite) / float32(torrent.Size)
 	}
 
-	now := time.Now()
-	bytes := torrent.t.BytesCompleted()
-	ulbytes := torrent.Stats.BytesWrittenData.Int64()
-
-	if !torrent.updatedAt.IsZero() {
+	if lastStat != nil {
 		// calculate rate
 		dtinv := float32(time.Second) / float32(now.Sub(torrent.updatedAt))
 
-		dldb := float32(bytes - torrent.Downloaded)
+		dldb := float32(bRead - lastStat.BytesReadUsefulData.Int64())
 		torrent.DownloadRate = dldb * dtinv
 
-		uldb := float32(ulbytes - torrent.Uploaded)
+		uldb := float32(bWrite - lastStat.BytesWrittenData.Int64())
 		torrent.UploadRate = uldb * dtinv
 	}
 
-	torrent.Downloaded = bytes
-	torrent.Uploaded = ulbytes
+	torrent.Downloaded = torrent.t.BytesCompleted()
+	torrent.Uploaded = bWrite
 	torrent.updatedAt = now
+	torrent.Stats = &curStat
 }
 
 func (torrent *Torrent) updateFileStatus() {
